/*
  net_portion.cc

  $Id: net_portion.c,v 1.2 1999/11/29 03:32:32 everton Exp $
 */

#include <syslog.h>
#include "util.h"
#include "addr.h"
#include "net_portion.h"

void net_portion::show() const
{
  show_addr(&address);
  syslog(LOG_INFO, "/%d", mask_len);
}

/*
 * Verifica se um endereco IP pertence `a rede.
 */
int net_portion::owns(const struct ip_addr *ip) const
{
  char  *net_a, *net_pastend;
  char  *ip_a;
  short len;
  short mod;

  /* 
   * Compara comprimento de endereco.
   */
  if (address.len != ip->len)
    return 0;

  /*
   * Compara todos os bytes, exceto possivelmente o ultimo
   * (quando o comprimento da mascara nao for multiplo de 8). 
   */
  len = mask_len >> 3; /* mask_len / 8 */
  net_a = address.addr;
  net_pastend = net_a + len;
  ip_a  = ip->addr;
  for (; net_a < net_pastend; ++net_a, ++ip_a)
    if (*net_a != *ip_a)
      return 0;

  /*
   * Se nao faltar parte do ultimo byte, o endereco ja' casou.
   *
   * mod = mask_len % 8 = numero de uns (1) `a esquerda
   */
  mod = mask_len & 7;
  if (!mod)
    return -1;

  /*
   * Compara ultimo byte.
   *
   * 8 - ()       => numero de zeros `a direta
   * 0xFF << ()   => insere zeros `a direta
   * *ip_a & ()   => desconsidera bits que sejam zero
   */
  return *net_a == (*ip_a & (0xFF << (8 - mod)));
}

/* Eof: net_portion.cc */
