/*
  proto_map.cc

  $Id: proto_map.c,v 1.1 1999/11/27 03:00:05 everton Exp $
 */

#include <unistd.h>
#include <syslog.h>
#include <errno.h>
#include "msignal.h"
#include "proto_map.hpp"
#include "forward.h"

void proto_map::show() const
{
  iterator<vector<int>,int> it(*port_list);
  it.start();
  if (it.cont()) {
    syslog(LOG_INFO, "%d", it.get());
    it.next();
  }
  for (; it.cont(); it.next())
    syslog(LOG_INFO, ", %d", it.get());

  syslog(LOG_INFO, " {\n");
  iterator<vector<host_map*>,host_map*> it2(*map_list);
  it2.start();
  if (it2.cont()) {
    it2.get()->show();
    it2.next();
  }
  for (; it2.cont(); it2.next()) {
    syslog(LOG_INFO, ";\n");
    it2.get()->show();
  }
  syslog(LOG_INFO, "\n}\n");
}

void proto_map::spawn_server(proto_t proto, int port) const
{
  pid_t pid = fork();
  if (pid) {
    if (pid < 0)
      syslog(LOG_ERR, "fork() failed: %s\n", strerror(errno));

    return;
  }

  /* Child */

  signal(SIGCHLD, SIG_IGN);

  switch (proto) {
  case P_TCP:
    tcp_forward(proto, port, map_list);
    break;
  case P_UDP:
    udp_forward(proto, port, map_list);
    break;
  default:
    syslog(LOG_ERR, "Unknown protocol identifier: %d\n", proto);
  }
    
  syslog(LOG_ERR, "Child exiting (!)\n");
  exit(1);
}

void proto_map::serve(proto_t proto) const
{
  if (!port_list) {
    syslog(LOG_WARNING, "Missing port list");
    return;
  }

  iterator<vector<int>,int> it(*port_list);
  for (it.start(); it.cont(); it.next())
    spawn_server(proto, it.get());
}

/* Eof: proto_map.cc */
